// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

// From dart-cli
T waitFor<T>(Future<T> future, {Duration? timeout}) {
  late T result;
  bool futureCompleted = false;
  Object? error;
  StackTrace? stacktrace;

  future.then(
    (T r) {
      futureCompleted = true;
      result = r;
    },
    onError: (Object? e, StackTrace? st) {
      error = e;
      stacktrace = st;
    },
  );

  late Stopwatch s;
  if (timeout != null) {
    s = Stopwatch()..start();
  }

  Timer.run(() {});

  while (!futureCompleted && (error == null)) {
    Duration? remaining;
    if (timeout != null) {
      if (s.elapsed >= timeout) {
        throw TimeoutException('waitFor() timed out', timeout);
      }
      remaining = timeout - s.elapsed;
    }
  }
  if (timeout != null) {
    s.stop();
  }

  Timer.run(() {});

  if (error != null) {
    throw AsyncError(error!, stacktrace);
  }

  return result;
}
